@@ -3,7 +3,7 @@
 ** distribution information.
 */
 
-/* $Id: acconfig.h,v 1.5 1999/12/13 03:34:04 mrsam Exp $ */
+/* $Id: acconfig.h,v 1.6 2000/04/26 22:32:37 mrsam Exp $ */
 
 /* Whether we must provide a prototype for crypt() */
 #undef	NEED_CRYPT_PROTOTYPE
@@ -28,3 +28,9 @@
 
 /* This macro is used to build the list of SASL modules */
 #undef	SASL_LIST
+
+/* This macro is used to build the real list of modules, for authdaemond */
+#undef	DAEMON_STATIC_LIST
+
+/* The REAL modules, used by authdaemond */
+#undef	AUTHDAEMONMODULELIST
