@@ -6,12 +6,13 @@
 #include	"auth.h"
 #include	"authstaticlist.h"
 
-static const char rcsid[]="$Id: authstaticlist.c,v 1.3 2000/02/20 21:48:36 mrsam Exp $";
+static const char rcsid[]="$Id: authstaticlist.c,v 1.4 2000/04/27 02:51:00 mrsam Exp $";
 
 /* First, prototype the functions */
 
 #define	AUTHSTATIC(name, func, prefunc, cleanupfunc) \
-	extern char *func(const char *, const char *, char *, int); \
+	extern char *func(const char *, const char *, char *, int, \
+			void (*)(struct authinfo *, void *), void *); \
 	extern int prefunc(const char *, const char *, \
 			int (*)(struct authinfo *, void *), void *arg); \
 	extern void cleanupfunc();
