@@ -15,7 +15,7 @@
 #endif
 #include	"auth.h"
 
-static const char rcsid[]="$Id: checkpassword.c,v 1.6 2000/06/29 01:46:59 mrsam Exp $";
+static const char rcsid[]="$Id: checkpassword.c,v 1.7 2000/09/03 00:41:09 mrsam Exp $";
 
 #if HAVE_CRYPT
 #if NEED_CRYPT_PROTOTYPE
@@ -31,10 +31,7 @@ int authcheckpassword(const char *password, const char *encrypted_password)
 {
 #if	HAVE_MD5LIB
 	if (strncmp(encrypted_password, "$1$", 3) == 0
-
-#if 0
-		|| strncmp(encrypted_password, "{MD5}", 5) == 0
-#endif
+		|| strncasecmp(encrypted_password, "{MD5}", 5) == 0
 		)
 		return (authcheckpasswordmd5(password, encrypted_password));
 #endif
