@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 1999 Double Precision, Inc.
+** Copyright 1998 - 2000 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -70,7 +70,7 @@
 
 #define	SHARED	"shared"
 
-static const char rcsid[]="$Id: imapd.c,v 1.35 2000/06/17 14:44:49 mrsam Exp $";
+static const char rcsid[]="$Id: imapd.c,v 1.44 2000/09/02 21:52:25 mrsam Exp $";
 
 extern time_t rfc822_parsedt(const char *);
 extern void fetchflags(unsigned long);
@@ -164,12 +164,6 @@ char	*q;
 	*q++='\0';
 	for (++p; (p=strtok(p, ",")) != 0; p=0)
 	{
-		if (strcmp(p, "imapdebug") == 0 && (p=getenv("IMAPDEBUGFILE"))
-			!= 0 && *p && debugfile == 0)
-		{
-			debugfile=fopen(p, "a");
-			continue;
-		}
 		if (strcmp(p, "allfolders") == 0)
 		{
 			flat_folders=1;
@@ -218,7 +212,11 @@ char	*arg;
 			errno=EINVAL;
 		}
 		else if (*arg == 0 || strchr(arg+1, '.') == 0)
-			arg="";
+		{
+			arg=malloc(1);
+			if (!arg)	write_error_exit(0);
+			*arg=0;
+		}
 		else
 			arg=maildir_shareddir(0, arg+1);
 	}
@@ -256,7 +254,7 @@ char	*arg;
 	return (arg);
 }
 
-static char *decode_valid_mailbox(char *p, int autosubscribe)
+char *decode_valid_mailbox(const char *p, int autosubscribe)
 {
 char	*q;
 char	*r;
@@ -273,8 +271,7 @@ char	*r;
 	if (access(r, 0))
 	{
 
-		if (autosubscribe && strncmp(r, SHAREDSUBDIR "/",
-			sizeof (SHAREDSUBDIR "/")-1) == 0)
+		if (autosubscribe && is_sharedsubdir(r))
 		{
 			/*
 			** automatically SUBSCRIBE to this folder, if needed
@@ -458,12 +455,14 @@ void maildir_mkfilename(const char *mailbox, struct imapflags *flags,
 char	*p;
 char	uniqbuf[80];
 static unsigned uniqcnt;
+int	rc;
 
 	sprintf(uniqbuf, "%u", uniqcnt++);
 
-	while (maildir_try_create_hostname(mailbox, uniqbuf, s, getenv("HOSTNAME"),
-		tmpname, newname))
+	while ((rc=maildir_try_create_hostname(mailbox, uniqbuf, s, getenv("HOSTNAME"),
+		tmpname, newname)))
 	{
+		if (rc < 0)	write_error_exit(0);
 		sleep(3);
 	}
 
@@ -476,8 +475,7 @@ static unsigned uniqcnt;
 		** a sharable folder, put it into cur
 		*/
 
-		strncmp(mailbox, SHAREDSUBDIR "/",
-			sizeof (SHAREDSUBDIR "/")-1))
+		!is_sharedsubdir(mailbox))
 			return;
 
 	/* Ok, this message will really go to cur, not new */
@@ -957,8 +955,7 @@ int	move_to_trash=0;
 		old_name=alloc_filename(current_mailbox,
 			current_mailbox_info.msgs[j].filename);
 
-		if (strncmp(current_mailbox, SHAREDSUBDIR "/",
-			sizeof(SHAREDSUBDIR "/")-1) == 0)
+		if (is_sharedsubdir(current_mailbox))
 		{
 			maildir_unlinksharedmsg(old_name);
 		}
@@ -996,6 +993,106 @@ int	move_to_trash=0;
 	}
 }
 
+static char *newsubscribefile()
+{
+char    *tmpname, *newname;
+char    uniqbuf[80];
+static  unsigned tmpuniqcnt=0;
+int	rc;
+
+	sprintf(uniqbuf, "imapsubscribe%u", tmpuniqcnt++);
+
+	while ((rc=maildir_try_create_hostname(".", uniqbuf,
+		0, getenv("HOSTNAME"), &tmpname, &newname)))
+	{
+		if (rc < 0)	write_error_exit(0);
+		sleep(3);
+	}
+
+	free(newname);
+	return (tmpname);
+}
+
+static int sub_strcmp(const char *a, const char *b)
+{
+	if (strncasecmp(a, "inbox", 5) == 0 &&
+		strncasecmp(b, "inbox", 5) == 0)
+	{
+		a += 5;
+		b += 5;
+	}
+	return (strcmp(a, b));
+}
+
+static void subscribe(const char *f)
+{
+char *newf=newsubscribefile();
+FILE *newfp=fopen(newf, "w");
+FILE *oldfp;
+
+	if (!newfp)
+		write_error_exit(0);
+
+	if ((oldfp=fopen(SUBSCRIBEFILE, "r")) != 0)
+	{
+	char	buf[BUFSIZ];
+
+		while (fgets(buf, sizeof(buf), oldfp) != 0)
+		{
+		char *p=strchr(buf, '\n');
+
+			if (p)	*p=0;
+			if (sub_strcmp(buf, f) == 0)
+			{
+				fclose(oldfp);
+				fclose(newfp);
+				unlink(newf);
+				free(newf);
+				return;	/* Already subscribed */
+			}
+			fprintf(newfp, "%s\n", buf);
+		}
+		fclose(oldfp);
+	}
+	fprintf(newfp, "%s\n", f);
+	if (fflush(newfp) || ferror(newfp))
+		write_error_exit(0);
+	fclose(newfp);
+	rename(newf, SUBSCRIBEFILE);
+	free(newf);
+}
+
+static void unsubscribe(const char *f)
+{
+char *newf=newsubscribefile();
+FILE *newfp=fopen(newf, "w");
+FILE *oldfp;
+
+	if (!newfp)
+		write_error_exit(0);
+
+	if ((oldfp=fopen(SUBSCRIBEFILE, "r")) != 0)
+	{
+	char	buf[BUFSIZ];
+
+		while (fgets(buf, sizeof(buf), oldfp) != 0)
+		{
+		char *p=strchr(buf, '\n');
+
+			if (p)	*p=0;
+			if (sub_strcmp(buf, f) == 0)
+				continue;
+			fprintf(newfp, "%s\n", buf);
+		}
+		fclose(oldfp);
+	}
+	if (fflush(newfp) || ferror(newfp))
+		write_error_exit(0);
+	fclose(newfp);
+	rename(newf, SUBSCRIBEFILE);
+	free(newf);
+}
+
 int do_imap_command(const char *tag)
 {
 struct	imaptoken *curtoken=nexttoken();
@@ -1393,20 +1490,44 @@ int	uid=0;
 	if (strcmp(curtoken->tokenbuf, "DELETE") == 0)
 	{
 	char	*mailbox;
+	char	*p;
+	char	*mailbox_name;
 
 		curtoken=nexttoken_nouc();
+
+		if (curtoken->tokentype != IT_NUMBER &&
+			curtoken->tokentype != IT_ATOM &&
+			curtoken->tokentype != IT_QUOTED_STRING)
+			return (-1);
+
+		p=strrchr(curtoken->tokenbuf, HIERCH);
+		if (p && p[1] == '\0')		/* Ignore hierarchy DELETE */
+		{
+			if (nexttoken()->tokentype != IT_EOL)
+				return (-1);
+			writes(tag);
+			writes(" OK Folder directory delete punted.\r\n");
+			return (0);
+		}
+
+		mailbox_name=my_strdup(curtoken->tokenbuf);
 		mailbox=parse_mailbox_error(tag, curtoken, 1, 0);
 		if ( mailbox == 0)
+		{
+			free(mailbox_name);
 			return (0);
+		}
 
 		if (nexttoken()->tokentype != IT_EOL)
 		{
+			free(mailbox_name);
 			free(mailbox);
 			return (-1);
 		}
 
 		if (current_mailbox && strcmp(mailbox, current_mailbox) == 0)
 		{
+			free(mailbox_name);
 			free(mailbox);
 			writes(tag);
 			writes(" NO Cannot delete selected folder.\r\n");
@@ -1416,6 +1537,7 @@ int	uid=0;
 		if (strncmp(curtoken->tokenbuf, SHARED HIERCHS,
 			sizeof(SHARED HIERCHS)-1) == 0)
 		{
+			free(mailbox_name);
 			free(mailbox);
 			writes(tag);
 			writes(" NO UNSUBSCRIBE, don't DELETE a shared folder.\r\n");
@@ -1426,11 +1548,18 @@ int	uid=0;
 		if (strcmp(mailbox, ".") == 0 ||
 			is_reserved(mailbox) || mddelete(mailbox))
 		{
+			free(mailbox_name);
 			free(mailbox);
 			writes(tag);
 			writes(" NO Cannot delete this folder.\r\n");
 			return (0);
 		}
+
+		// Automatically UNSUBSCRIBE a DELETEd mailbox
+
+		unsubscribe(mailbox_name);
+
+		free(mailbox_name);
 		free(mailbox);
 		writes(tag);
 		writes(" OK Folder deleted.\r\n");
@@ -1559,8 +1688,7 @@ int	uid=0;
 
 		/* check if this is a shared read-only folder */
 
-		if (strncmp(mailbox, SHAREDSUBDIR "/",
-				sizeof(SHAREDSUBDIR "/")-1) == 0 &&
+		if (is_sharedsubdir(mailbox) &&
 			maildir_sharedisro(mailbox))
 			ro=1;
 
@@ -1596,6 +1724,7 @@ int	uid=0;
 	{
 	char	*mailbox;
 	char	*dir;
+	char	*p;
 
 		curtoken=nexttoken_nouc();
 		if (curtoken->tokentype != IT_NUMBER &&
@@ -1603,20 +1732,31 @@ int	uid=0;
 			curtoken->tokentype != IT_QUOTED_STRING)
 			return (-1);
 
-		mailbox=my_strdup(curtoken->tokenbuf);
-
-		if (strncmp(mailbox, SHARED HIERCHS, sizeof(SHARED HIERCHS)-1))
+		p=strrchr(curtoken->tokenbuf, HIERCH);
+		if (p && p[1] == '\0')		/* Ignore hierarchy DELETE */
 		{
-			free(mailbox);
+			if (nexttoken()->tokentype != IT_EOL)
+				return (-1);
 			writes(tag);
-			writes(" OK SUBSCRIBE is not necessary.\r\n");
+			writes(" OK Folder directory subscribe punted.\r\n");
 			return (0);
 		}
 
+		mailbox=my_strdup(curtoken->tokenbuf);
 		if (nexttoken()->tokentype != IT_EOL)
 			return (-1);
 
 		dir=valid_mailbox_name(mailbox, 1);
+
+		if (dir && !is_sharedsubdir(dir))
+		{
+			subscribe(mailbox);
+			free(mailbox);
+			free(dir);
+			writes(tag);
+			writes(" OK Folder subscribed.\r\n");
+			return (0);
+		}
 		if (dir && (*dir == 0 || access(dir, 0) == 0))
 		{
 			free(dir);
@@ -1643,6 +1783,7 @@ int	uid=0;
 	if (strcmp(curtoken->tokenbuf, "UNSUBSCRIBE") == 0)
 	{
 	char	*mailbox, *dir;
+	char	*p;
 
 		curtoken=nexttoken_nouc();
 		if (curtoken->tokentype != IT_NUMBER &&
@@ -1650,20 +1791,36 @@ int	uid=0;
 			curtoken->tokentype != IT_QUOTED_STRING)
 			return (-1);
 
-		mailbox=my_strdup(curtoken->tokenbuf);
-
-		if (strncmp(mailbox, SHARED HIERCHS, sizeof(SHARED HIERCHS)-1))
+		p=strrchr(curtoken->tokenbuf, HIERCH);
+		if (p && p[1] == '\0')		/* Ignore hierarchy DELETE */
 		{
-			free(mailbox);
+			if (nexttoken()->tokentype != IT_EOL)
+				return (-1);
 			writes(tag);
-			writes(" OK UNSUBSCRIBE is not necessary.\r\n");
+			writes(" OK Folder directory unsubscribe punted.\r\n");
 			return (0);
 		}
 
+		mailbox=my_strdup(curtoken->tokenbuf);
+
 		if (nexttoken()->tokentype != IT_EOL)
+		{
+			free(mailbox);
 			return (-1);
+		}
 
 		dir=valid_mailbox_name(mailbox, 1);
+
+		if (dir && !is_sharedsubdir(dir))
+		{
+			unsubscribe(mailbox);
+			free(mailbox);
+			free(dir);
+			writes(tag);
+			writes(" OK Folder unsubscribed.\r\n");
+			return (0);
+		}
+
 		if (dir && (*dir == 0 || access(dir, 0)))
 		{
 			free(dir);
@@ -1908,6 +2065,11 @@ int	uid=0;
 			thread_func=dothreadorderedsubj;
 			thread_type=search_orderedsubj;
 		}
+		else if (strcmp(curtoken->tokenbuf, "REFERENCES") == 0)
+		{
+			thread_func=dothreadreferences;
+			thread_type=search_references1;
+		}
 		else
 		{
 			return (-1);
@@ -2035,7 +2197,7 @@ int	uid=0;
 			}
 
 			newts=(struct temp_sort_stack *)malloc(
-				sizeof(struct temp_sort_stack *));
+				sizeof(struct temp_sort_stack));
 			if (!newts)	write_error_exit(0);
 			newts->next=ts;
 			newts->type=st;
@@ -2203,8 +2365,7 @@ int	uid=0;
 		*/
 
 		isshared=0;
-		if (strncmp(cqinfo.destmailbox, SHAREDSUBDIR "/",
-			sizeof( SHAREDSUBDIR "/")-1) == 0)
+		if (is_sharedsubdir(cqinfo.destmailbox))
 		{
 		char	*p=malloc(strlen(cqinfo.destmailbox)+sizeof("/shared"));
 
@@ -2356,6 +2517,8 @@ const char *tag;
 	ip=getenv("TCPREMOTEIP");
 	if (!ip || !*ip)	exit(0);
 
+	putenv("IMAP_STARTTLS=NO");	/* No longer grok STARTTLS */
+
 	fprintf(stderr, "INFO: LOGIN, user=%s, ip=[%s]\n",
 		getenv("AUTHENTICATED"), ip);
 
@@ -2390,6 +2553,9 @@ const char *tag;
 	mdcreate("." DRAFTS);
 #endif
 
+	if ((p=getenv("IMAPDEBUGFILE")) != 0 && *p)
+		debugfile=fopen(p, "a");
+
 	emptytrash();
 	if ((tag=getenv("IMAPLOGINTAG")) != 0)
 	{
