@@ -55,7 +55,7 @@
 #include	"imapwrite.h"
 #include	"imapd.h"
 
-static const char rcsid[]="$Id: imapscanclient.c,v 1.10 2000/04/28 01:51:13 mrsam Exp $";
+static const char rcsid[]="$Id: imapscanclient.c,v 1.12 2000/08/20 19:01:34 mrsam Exp $";
 
 static int do_imapscan_maildir2(struct imapscaninfo *, const char *, int);
 
@@ -224,8 +224,7 @@ struct	dirent *de;
 unsigned long left_unseen=0;
 int	dowritecache=0;
 
-	if (strncmp(dir, SHAREDSUBDIR "/",
-		sizeof (SHAREDSUBDIR "/")-1) == 0)
+	if (is_sharedsubdir(dir))
 		maildir_shared_sync(dir);
 
 
@@ -246,12 +245,14 @@ int	dowritecache=0;
 	char	*tmpname, *newname;
 	char	uniqbuf[80];
 	static  unsigned tmpuniqcnt;
+	int	rc;
 
 		sprintf(uniqbuf, "imapuid_%u", tmpuniqcnt++);
 
-		while (maildir_try_create_hostname(".", uniqbuf,
-			0, getenv("HOSTNAME"), &tmpname, &newname))
+		while ((rc=maildir_try_create_hostname(".", uniqbuf,
+			0, getenv("HOSTNAME"), &tmpname, &newname)) != 0)
 		{
+			if (rc < 0)	write_error_exit(0);
 			sleep(3);
 		}
 
