@@ -13,7 +13,7 @@
 #include	<stdlib.h>
 #include	<string.h>
 
-static const char rcsid[]="$Id: msgenvelope.c,v 1.4 1999/12/09 23:52:30 mrsam Exp $";
+static const char rcsid[]="$Id: msgenvelope.c,v 1.5 2000/07/30 01:08:09 mrsam Exp $";
 
 #define	MAX_HEADER_SIZE	8192
 
@@ -109,13 +109,13 @@ char	*q, *r;
 	if (!t)
 	{
 		perror("malloc");
-		_exit(0);
+		exit(0);
 	}
 	a=rfc822a_alloc(t);
 	if (!a)
 	{
 		perror("malloc");
-		_exit(1);
+		exit(1);
 	}
 
 	if (a->naddrs == 0)
@@ -139,7 +139,7 @@ char	*q, *r;
 		if (!q)
 		{
 			perror("malloc");
-			_exit(1);
+			exit(1);
 		}
 		if (a->addrs[i].tokens == 0)
 		{
@@ -165,7 +165,7 @@ char	*q, *r;
 		if (!q)
 		{
 			perror("malloc");
-			_exit(1);
+			exit(1);
 		}
 		r=strrchr(q, '@');
 		if (r)	*r++=0;
@@ -200,7 +200,7 @@ char	*p, *q, *r;
 	if (fseek(fp, start_pos, SEEK_SET) < 0)
 	{
 		perror("fseek");
-		_exit(0);
+		exit(0);
 	}
 
 	while ((p=read_header(fp, &start_pos, &start_body)) != 0)
@@ -233,7 +233,7 @@ char	*p, *q, *r;
 		if (!r)
 		{
 			perror("malloc");
-			_exit(1);
+			exit(1);
 		}
 		if (oldl && oldl < c)
 			r[oldl++]=',';
@@ -251,7 +251,7 @@ char	*p, *q, *r;
 	if (!replyto || !sender)
 	{
 		perror("malloc");
-		_exit(1);
+		exit(1);
 	}
 
 	(*writefunc)("(", 1);
