@@ -18,6 +18,7 @@
 #include	<sys/types.h>
 #include	<sys/stat.h>
 #include	"rfc822/rfc822.h"
+#include	"rfc822/rfc2047.h"
 #include	"rfc2045/rfc2045.h"
 #include	"numlib/numlib.h"
 #include	"searchinfo.h"
@@ -25,7 +26,7 @@
 #include	"imaptoken.h"
 #include	"imapscanclient.h"
 
-static const char rcsid[]="$Id: search.c,v 1.5 2000/06/09 12:29:14 mrsam Exp $";
+static const char rcsid[]="$Id: search.c,v 1.12 2000/09/02 23:55:55 mrsam Exp $";
 
 extern time_t rfc822_parsedt(const char *);
 extern struct imapscaninfo current_mailbox_info;
@@ -35,9 +36,11 @@ extern int get_flagname(const char *, struct imapflags *);
 extern void get_message_flags( struct imapscanmessageinfo *,
 	char *, struct imapflags *);
 
+static void fill_search_veryquick(struct searchinfo *,
+	unsigned long, struct imapflags *);
 
 static void fill_search_quick(struct searchinfo *,
-	unsigned long, struct imapflags *, struct stat *);
+	unsigned long, struct stat *);
 
 static void fill_search_header(struct searchinfo *, struct rfc2045 *, FILE *,
 	struct imapscanmessageinfo *);
@@ -95,6 +98,21 @@ int	rc;
 		for (p=sihead; p; p=p->next)
 			p->value= -1;	/* Search result unknown */
 
+		/* First, see if non-content search will be sufficient */
+
+		get_message_flags(current_mailbox_info.msgs+i, 0, &flags);
+
+		for (p=sihead; p; p=p->next)
+			fill_search_veryquick(p, i, &flags);
+
+		if ((rc=search_evaluate(si)) >= 0)
+		{
+			if (rc > 0)
+				(*callback_func)(si, sihead, isuid, i,
+						 voidarg);
+			continue;
+		}
+
 		fd=imapscan_openfile(current_mailbox,
 			&current_mailbox_info, i);
 		if (fd < 0)	continue;
@@ -102,7 +120,6 @@ int	rc;
 		if ((fp=fdopen(fd, "r")) == 0)
 			write_error_exit(0);
 
-		get_message_flags(current_mailbox_info.msgs+i, 0, &flags);
 		if (fstat(fileno(fp), &stat_buf))
 		{
 			fclose(fp);
@@ -112,7 +129,7 @@ int	rc;
 		/* First, see if non-content search will be sufficient */
 
 		for (p=sihead; p; p=p->next)
-			fill_search_quick(p, i, &flags, &stat_buf);
+			fill_search_quick(p, i, &stat_buf);
 
 		if ((rc=search_evaluate(si)) < 0)
 		{
@@ -239,19 +256,10 @@ char	buf[200];
 
 /* Evaluate non-content search nodes */
 
-static void fill_search_quick(struct searchinfo *p,
-	unsigned long msgnum, struct imapflags *flags, struct stat *stat_buf)
+static void fill_search_veryquick(struct searchinfo *p,
+	unsigned long msgnum, struct imapflags *flags)
 {
-	switch (p->type)	{
-	case search_messageset:
-		if (is_in_set(p->as, msgnum+1))
-			p->value=1;
-		else
-			p->value=0;
-		break;
-	case search_all:
-		p->value=1;
-		break;
+	switch (p->type) {
 	case search_keyword:
 		{
 		struct imapflags f;
@@ -272,6 +280,31 @@ static void fill_search_quick(struct searchinfo *p,
 				p->value=1;
 		}
 		break;
+	case search_messageset:
+		if (is_in_set(p->as, msgnum+1))
+			p->value=1;
+		else
+			p->value=0;
+		break;
+	case search_all:
+		p->value=1;
+		break;
+	case search_uid:
+		if (is_in_set(p->as, current_mailbox_info.msgs[msgnum].uid))
+			p->value=1;
+		break;
+	case search_reverse:
+		p->value=1;
+		break;
+	default:
+		break;
+	}
+}
+
+static void fill_search_quick(struct searchinfo *p,
+	unsigned long msgnum, struct stat *stat_buf)
+{
+	switch (p->type)	{
 	case search_before:
 		p->value=0;
 		{
@@ -319,11 +352,11 @@ static void fill_search_quick(struct searchinfo *p,
 				p->value=1;
 		}
 		break;
-	case search_uid:
-		if (is_in_set(p->as, current_mailbox_info.msgs[msgnum].uid))
-			p->value=1;
-		break;
 	case search_orderedsubj:
+	case search_references1:
+	case search_references2:
+	case search_references3:
+	case search_references4:
 	case search_arrival:
 	case search_cc:
 	case search_date:
@@ -341,6 +374,12 @@ static void fill_search_quick(struct searchinfo *p,
 			p->as=0;
 		}
 
+		if (p->bs)
+		{
+			free(p->bs);
+			p->bs=0;
+		}
+
 		switch (p->type)	{
 		case search_arrival:
 			p->as=timestamp_for_sorting(stat_buf->st_mtime);
@@ -352,16 +391,13 @@ static void fill_search_quick(struct searchinfo *p,
 			char *q;
 
 				str_size_t(stat_buf->st_size, buf);
-				sprintf(buf2, "%*s", sizeof(buf2)-1, buf);
+				sprintf(buf2, "%*s", (int)(sizeof(buf2)-1), buf);
 				for (q=buf2; *q == ' '; *q++='0')
 					;
 				p->as=my_strdup(buf2);
 				p->value=1;
 			}
 			break;
-		case search_reverse:
-			p->value=1;
-			break;
 		default:
 			break;
 		}
@@ -382,7 +418,11 @@ static int search_evaluate(struct searchinfo *si)
 int	rc, rc2;
 
 	switch (si->type)	{
-	case search_orderedsubj:	/* DUMMY for THREAD and SORT */
+	case search_orderedsubj:	/* DUMMIES for THREAD and SORT */
+	case search_references1:
+	case search_references2:
+	case search_references3:
+	case search_references4:
         case search_arrival:
         case search_cc:
         case search_date:
@@ -417,89 +457,6 @@ int	rc, rc2;
 	return (rc);
 }
 
-/* Remove artifacts from the subject header */
-
-static void stripsubj(char *s)
-{
-char	*p;
-char	*q;
-
-	for (p=s; *p; p++)
-		;
-	while (p > s)
-	{
-		if ( isspace((int)(unsigned char)p[-1]))
-		{
-			--p;
-			continue;
-		}
-		if (p-s >= 3 && strcmp(p-3, "(FWD)") == 0)
-		{
-			p -= 3;
-			continue;
-		}
-		break;
-	}
-	*p=0;
-
-	for (p=s; *p; )
-	{
-		for (;;)
-		{
-			if (isspace((int)(unsigned char)*p))
-			{
-				++p;
-				continue;
-			}
-			if (strncmp(p, "RE", 2) == 0)
-			{
-				q=p+2;
-			}
-			else if (strncmp(p, "FWD", 3) == 0)
-			{
-				q=p+3;
-			}
-			else if (strncmp(p, "FW", 2) == 0)
-			{
-				q=p+2;
-			}
-			else	break;
-
-			if (*q == '[')
-			{
-				++q;
-				if (!isdigit((int)(unsigned char)*q))
-					break;
-				do
-				{
-					++q;
-				} while (isdigit((int)(unsigned char)*q));
-				if (*q != ']')	break;
-				++q;
-			}
-			if (*q++ != ':')	break;
-
-			while (*q && isspace((int)(unsigned char)*q))
-				++q;
-			p=q;
-		}
-
-		if (*p != '[')	break;
-		for (q=p; *q; q++)
-			if (*q == ']')
-				break;
-		if (*q != ']')	break;
-		++q;
-		while (*q && isspace((int)(unsigned char)*q))
-			++q;
-		p=q;
-	}
-
-	q=s;
-	while ( (*q++ = *p++) != 0)
-		;
-}
-
 /* ------- header search -------- */
 
 static void fill_search_header_recursive(struct searchinfo *,
@@ -508,9 +465,24 @@ static void fill_search_header_recursive(struct searchinfo *,
 static void fill_search_header(struct searchinfo *si,
 	struct rfc2045 *rfcp, FILE *fp, struct imapscanmessageinfo *mi)
 {
-off_t start_pos, end_pos, start_body;
-off_t nlines, nbodylines;
-struct searchinfo *sip;
+	off_t start_pos, end_pos, start_body;
+	off_t nlines, nbodylines;
+	struct searchinfo *sip;
+
+	/* Consider the following dummy nodes as evaluated */
+
+	for (sip=si; sip; sip=sip->next)
+		switch (sip->type) {
+		case search_orderedsubj:
+		case search_references1:
+		case search_references2:
+		case search_references3:
+		case search_references4:
+			sip->value=1;
+			break;
+		default:
+			break;
+		}
 
 	rfc2045_mimepos(rfcp, &start_pos, &end_pos, &start_body,
 		&nlines, &nbodylines);
@@ -578,6 +550,14 @@ struct searchinfo *sip;
 			if (isdate == 0) switch (sip->type)	{
 			time_t given_time, msg_time;
 
+			case search_orderedsubj:
+				if (sip->bs == 0)
+				{
+					sip->bs=strdup(q);
+					if (!sip->bs)
+						write_error_exit(0);
+				}
+				break;
 			case search_sentbefore:
 			case search_sentsince:
 			case search_senton:
@@ -659,15 +639,63 @@ struct searchinfo *sip;
 				break;
 			}
 
+			if (sip->type == search_references1 &&
+			    strcmp(p, "REFERENCES") == 0 && sip->as == 0)
+			{
+				sip->as=strdup(q);
+				if (!sip->as)
+					write_error_exit(0);
+			}
+
+			if (sip->type == search_references1 &&
+			    strcmp(p, "IN-REPLY-TO") == 0 && sip->bs == 0)
+			{
+				sip->bs=strdup(q);
+				if (!sip->bs)
+					write_error_exit(0);
+			}
+
+			if (sip->type == search_references2 &&
+			    strcmp(p, "DATE") == 0 && sip->as == 0)
+			{
+				sip->as=strdup(q);
+				if (!sip->as)
+					write_error_exit(0);
+			}
+
+			if (sip->type == search_references3 &&
+			    strcmp(p, "SUBJECT") == 0 && sip->as == 0)
+			{
+				sip->as=strdup(q);
+				if (!sip->as)
+					write_error_exit(0);
+			}
+
+			if (sip->type == search_references4 &&
+			    strcmp(p, "MESSAGE-ID") == 0 && sip->as == 0)
+			{
+				sip->as=strdup(q);
+				if (!sip->as)
+					write_error_exit(0);
+			}
+
 			if (sip->type == search_orderedsubj &&
 				strcmp(p, "SUBJECT") == 0 && sip->as == 0)
 			{
+				int dummy;
+				char *r;
+
 			/* Dummy node used to suck in the Subject:
 			** header for the THREAD or SORT command.
 			*/
-				sip->as=my_strdup(q);
-				sip->value=1;
-				stripsubj(sip->as);
+				r=rfc2047_decode_simple(q);
+				if (!r)
+					write_error_exit(0);
+
+				sip->as=rfc822_coresubj(r, &dummy);
+				free(r);
+				if (!sip->as)
+					write_error_exit(0);
 			}
 
 			if (sip->type != search_header ||
