@@ -14,7 +14,7 @@
 #include	"imapwrite.h"
 #include	"imaptoken.h"
 
-static const char rcsid[]="$Id: searchinfo.c,v 1.4 2000/06/09 12:29:14 mrsam Exp $";
+static const char rcsid[]="$Id: searchinfo.c,v 1.5 2000/09/02 21:52:25 mrsam Exp $";
 
 
 static struct searchinfo *alloc_search(
@@ -65,8 +65,17 @@ struct	searchinfo *si;
 struct searchinfo *alloc_searchextra(struct searchinfo *top,
 	struct searchinfo **head, search_type t)
 {
-struct searchinfo *si=alloc_search(head);
+	struct searchinfo *si;
 
+	if (t == search_references1)
+	{
+		/* Automatically add third and second dummy node */
+
+		top=alloc_searchextra(top, head, search_references4);
+		top=alloc_searchextra(top, head, search_references3);
+		top=alloc_searchextra(top, head, search_references2);
+	}
+	si=alloc_search(head);
 	si->type=t;
 	si->a=top;
 	return (si);
@@ -599,7 +608,34 @@ void debug_search(struct searchinfo *si)
 		writes("\"");
 		return;
 	case search_orderedsubj:
-		writes("ORDEREDSUBJ");
+		writes("ORDEREDSUBJ ");
+		debug_search(si->a);
+		return;
+	case search_references1:
+		writes("REFERENCES[References/In-Reply-To]=");
+		writeqs(si->as ? si->as:"");
+		writes("/");
+		writeqs(si->bs ? si->bs:"");
+		writes(" ");
+		debug_search(si->a);
+		return;
+	case search_references2:
+		writes("REFERENCES[Date:]=");
+		writeqs(si->as ? si->as:"");
+		writes(" ");
+		debug_search(si->a);
+		return;
+	case search_references3:
+		writes("REFERENCES[Subject]=");
+		writeqs(si->as ? si->as:"");
+		writes(" ");
+		debug_search(si->a);
+		return;
+	case search_references4:
+		writes("REFERENCES[Message-ID]=");
+		writeqs(si->as ? si->as:"");
+		writes(" ");
+		debug_search(si->a);
 		return;
 	case search_arrival:
 		writes("ARRIVAL");
