@@ -6,7 +6,7 @@
 ** See COPYING for distribution information.
 */
 
-static const char searchinfo_h_rcsid[]="$Id: searchinfo.h,v 1.3 2000/06/09 12:29:14 mrsam Exp $";
+static const char searchinfo_h_rcsid[]="$Id: searchinfo.h,v 1.4 2000/09/02 21:52:25 mrsam Exp $";
 
 	/* Search keys */
 
@@ -48,6 +48,16 @@ typedef enum {
 
 	search_orderedsubj,
 
+	/*
+	** search_references? are dummy nodes that are allocated in order to
+	** implement a REFERENCES THREAD.
+	*/
+
+	search_references1,	/*  References: and In-Reply-To: header */
+	search_references2,	/*  Date: header */
+	search_references3,	/*  Subject: header */
+	search_references4,	/*  Message-ID: header */
+
 	/*
 	** And the following dummies are used for similar purposes for the
 	** SORT command.
