@@ -18,7 +18,7 @@
 #include	<stdlib.h>
 #include	"numlib/numlib.h"
 
-static const char rcsid[]="$Id: maildircreateh.c,v 1.4 2000/02/07 13:39:01 mrsam Exp $";
+static const char rcsid[]="$Id: maildircreateh.c,v 1.5 2000/05/24 03:42:00 mrsam Exp $";
 
 int maildir_try_create_hostname(const char *m,	/* Maildir */
 	const char *u,	/* Unique when creating multiple msgs by one pid */
@@ -57,7 +57,7 @@ char	pbuf[NUMBUFSIZE];
 	strcpy( (*nptr), (*tptr) );
 	memcpy( (*nptr)+l+1, "new", 3);
 
-	if (stat( (*nptr), &stat_buf) < 0 && errno == ENOENT)
+	if (stat( (*nptr), &stat_buf) < 0)
 		return (0);
 	free( *nptr );
 	free( *tptr );
