@@ -10,6 +10,7 @@
 #include	<sys/stat.h>
 #include	<time.h>
 #include	<stdio.h>
+#include	<errno.h>
 #include	<string.h>
 #if	HAVE_STRINGS_H
 #include	<strings.h>
@@ -35,7 +36,7 @@ int gethostname(const char *, size_t);
 #endif
 
 
-static const char rcsid[]="$Id: reformime.c,v 1.23 2000/05/23 17:55:02 mrsam Exp $";
+static const char rcsid[]="$Id: reformime.c,v 1.29 2000/07/27 22:09:45 mrsam Exp $";
 
 void rfc2045_error(const char *errmsg)
 {
@@ -91,6 +92,7 @@ void usage()
 static char *tempname(const char *tempdir)
 {
 char	pidbuf[NUMBUFSIZE], timebuf[NUMBUFSIZE], hostnamebuf[256];
+static unsigned counter=0;
 time_t	t;
 char	*p;
 
@@ -101,9 +103,10 @@ char	*p;
 	if (gethostname(hostnamebuf, sizeof(hostnamebuf)))
 		hostnamebuf[0]=0;
 	p=malloc(strlen(tempdir)+strlen(pidbuf)+strlen(timebuf)+
-		strlen(hostnamebuf)+4);
+		strlen(hostnamebuf)+100);
 	if (!p)	return (0);
-	sprintf(p, "%s/%s.%s.%s", tempdir, timebuf, pidbuf, hostnamebuf);
+	sprintf(p, "%s/%s.%s-%u.%s", tempdir, timebuf, pidbuf, counter++,
+		hostnamebuf);
 	return (p);
 }
 
@@ -111,67 +114,31 @@ struct rfc2045 *read_message()
 {
 char	buf[BUFSIZ];
 struct	rfc2045 *p=rfc2045_alloc_ac();
-int	tempfd= -1;
+FILE	*tempfp=0;
 
 	if (fseek(stdin, 0L, SEEK_END) < 0 ||
 		fseek(stdin, 0L, SEEK_SET) < 0)	/* Pipe, save to temp file */
 	{
-	struct passwd *p;
-	char	*tmpdir, *tmpfilename;
-
-		/* Create temp file in $HOME/.tmp */
-
-		p=getpwuid(getuid());
-		if (!p)
-		{
-			fprintf(stderr, "reformime: unable to read your home directory.\n");
-			exit(1);
-		}
-		tmpdir=malloc(strlen(p->pw_dir)+6);
-		if (!tmpdir ||
-			!(tmpfilename=tempname(strcat(strcpy(tmpdir,
-						p->pw_dir), "/.tmp"))))
-		{
-			perror("malloc");
-			exit(1);
-		}
-		mkdir(tmpdir, 0700);
-		if ((tempfd=open(tmpfilename, O_RDWR|O_CREAT, 0600)) < 0)
-		{
-			perror("open");
-		}
-		unlink(tmpfilename);
-		free(tmpdir);
-		free(tmpfilename);
+		tempfp=tmpfile();
 	}
 
 	while (fgets(buf, sizeof(buf), stdin))
 	{
-		rfc2045_parse(p, buf, strlen(buf));
-		if (tempfd >= 0)
+	int l=strlen(buf);
+
+		rfc2045_parse(p, buf, l);
+		if (tempfp && fwrite(buf, l, 1, tempfp) != 1)
 		{
-		const char *p=buf;
-		size_t l=strlen(p);
-
-			while (l)
-			{
-			int n=write(tempfd, p, l);
-
-				if (n <= 0)
-				{
-					perror("write");
-					exit(1);
-				}
-				p += n;
-				l -= n;
-			}
+			perror("fwrite");
+			exit(1);
 		}
 	}
-	if (tempfd >= 0)
+
+	if (tempfp)	/* Replace stdin */
 	{
 		close(0);
-		dup(tempfd);
-		close(tempfd);
+		dup(fileno(tempfp));
+		fclose(tempfp);
 	}
 	return (p);
 }
@@ -314,7 +281,8 @@ void rewrite(struct rfc2045 *p, int rwmode)
 	}
 }
 
-static char *get_suitable_filename(struct rfc2045 *r, const char *pfix)
+static char *get_suitable_filename(struct rfc2045 *r, const char *pfix,
+	int ignore_filename)
 {
 const char *disposition_s;
 const char *disposition_name_s;
@@ -333,7 +301,24 @@ char	*dyn_disp_name=0;
 	if (!disposition_filename_s || !*disposition_filename_s)
 		disposition_filename_s=content_name_s;
 
-	if (!disposition_filename_s || !*disposition_filename_s)
+	if (ignore_filename)
+	{
+	char	numbuf[NUMBUFSIZE];
+	static size_t counter=0;
+	const char *p=str_size_t(++counter, numbuf);
+
+		dyn_disp_name=malloc(strlen(disposition_filename_s)
+			+ strlen(p)+2);
+		if (!dyn_disp_name)
+		{
+			perror("malloc");
+			exit(1);
+		}
+		disposition_filename_s=strcat(strcat(strcpy(
+			dyn_disp_name, p), "-"),
+			disposition_filename_s);
+	}
+	else if (!disposition_filename_s || !*disposition_filename_s)
 	{
 		dyn_disp_name=tempname(".");
 		disposition_filename_s=dyn_disp_name+2;	/* Skip over ./ */
@@ -416,15 +401,38 @@ char	*dyn_disp_name=0;
 static void extract_file(struct rfc2045 *p,
 	const char *filename, int argc, char **argv)
 {
-char	*f=get_suitable_filename(p, filename);
+char	*f;
 FILE	*fp;
+int	ignore=0;
 
-	if (!f)	return;
-
-	if ((fp=fopen(f, "w")) == 0)
+	for (;;)
 	{
-		perror(f);
-		exit(1);
+	int	fd;
+
+		f=get_suitable_filename(p, filename, ignore);
+		if (!f)	return;
+
+		fd=open(f, O_WRONLY|O_CREAT|O_EXCL, 0666);
+		if (fd < 0)
+		{
+			if (errno == EEXIST)
+			{
+				printf("%s exists.\n", f);
+				free(f);
+				ignore=1;
+				continue;
+			}
+
+			perror(f);
+			exit(1);
+		}
+		fp=fdopen(fd, "w");
+		if (!fp)
+		{
+			perror("fdopen");
+			exit(1);
+		}
+		break;
 	}
 
 	do_print_section(p, fp);
@@ -441,7 +449,7 @@ static void extract_pipe(struct rfc2045 *p,
 	const char *filename,
 	int argc, char **argv)
 {
-char	*f=get_suitable_filename(p, "FILENAME=");
+char	*f=get_suitable_filename(p, "FILENAME=", 0);
 int	pipefd[2];
 pid_t	pid, p2;
 FILE	*fp;
@@ -855,6 +863,9 @@ const char *extract_filename=0;
 		case 'X':
 			do_extract=extract_pipe;
 			break;
+		case 'v':
+			printf("%s\n", rcsid);
+			exit(0);
 		default:
 			usage();
 		}
