@@ -4,7 +4,7 @@
 */
 
 /*
-** $Id: rfc2045.c,v 1.20 2000/04/10 22:42:02 mrsam Exp $
+** $Id: rfc2045.c,v 1.21 2000/08/05 21:56:07 mrsam Exp $
 */
 #if    HAVE_CONFIG_H
 #include       "config.h"
@@ -35,7 +35,11 @@ struct rfc2045 *rfc2045_alloc()
 {
 struct rfc2045 *p=(struct rfc2045 *)malloc(sizeof(struct rfc2045));
 
-	if (!p)	rfc2045_enomem();
+	if (!p)
+	{
+		rfc2045_enomem();
+		return (0);
+	}
 
 	/* Initialize everything to nulls, except for one thing */
 
@@ -83,13 +87,19 @@ char	*v;
 	}
 
 	v=strdup(val);
-	if (!v)	rfc2045_enomem();
+	if (!v)
+	{
+		rfc2045_enomem();
+		return;
+	}
+
 	if (!*p)
 	{
 		if (((*p)=(struct rfc2045attr *)malloc(sizeof(**p))) == 0)
 		{
 			free(v);
 			rfc2045_enomem();
+			return;
 		}
 		memset( (*p), 0, sizeof(**p));
 		if ( ((*p)->name=strdup(name)) == 0)
@@ -98,6 +108,7 @@ char	*v;
 			*p=0;
 			free(v);
 			rfc2045_enomem();
+			return;
 		}
 	}
 	if ( (*p)->value )	free ( (*p)->value );
@@ -174,7 +185,11 @@ void rfc2045_add_buf(
 	char	*p= *bufptr ? (char *)realloc(*bufptr, newsize):
 				(char *)malloc(newsize);
 
-		if (!p)	rfc2045_enomem();
+		if (!p)
+		{
+			rfc2045_enomem();
+			return;
+		}
 		*bufptr=p;
 		*bufsize=newsize;
 	}
@@ -214,7 +229,10 @@ static void set_string(char **p,
 	if (!q)	return;
 
 	if ((*p=(char *)malloc(strlen(q)+1)) == 0)
+	{
 		rfc2045_enomem();
+		return;
+	}
 
 	strcpy(*p, q);
 }
@@ -672,12 +690,8 @@ char	*p;
 	p=( char *)malloc(l);
 	if (!p)
 	{
-		/* The RFC822 structure we've allocated was temporary.
-		** Error condition - free it, so we don't leak memory.
-		*/
-
-		rfc822t_free(h);
 		rfc2045_enomem();
+		return (0);
 	}
 	l=0;
 
@@ -708,7 +722,7 @@ static char *lower_paste_tokens(struct rfc822t *h, int start, int cnt)
 char	*p=paste_tokens(h, start, cnt);
 char	*q;
 
-	for (q=p; *q; q++)
+	for (q=p; q && *q; q++)
 		*q=tolower(*q);
 	return (p);
 }
@@ -724,7 +738,7 @@ static char *lower_paste_token(struct rfc822t *h, int i)
 char *p=paste_token(h, i);
 char *q;
 
-	for (q=p; *q; q++)
+	for (q=p; q && *q; q++)
 		*q=tolower(*q);
 	return (p);
 }
@@ -768,7 +782,10 @@ char	*t;
 	}
 
 	t=lower_paste_token(header, 0);
-	if (strcmp(t, "mime-version") == 0)
+
+	if (t == 0)
+		;
+	else if (strcmp(t, "mime-version") == 0)
 	{
 		free(t);
 		mime_version(p, header);
@@ -832,6 +849,8 @@ static void mime_version(struct rfc2045 *p, struct rfc822t *header)
 {
 char	*vers=paste_tokens(header, 2, header->ntokens-2);
 
+	if (!vers)	return;
+
 	if (p->mime_version)	free(p->mime_version);
 	p->mime_version=vers;
 }
@@ -842,6 +861,8 @@ static void content_transfer_encoding(struct rfc2045 *r,
 char	*p;
 
 	p=lower_paste_tokens(header, 2, header->ntokens-2);
+	if (!p)	return;
+
 	if (r->content_transfer_encoding)
 		free(r->content_transfer_encoding);
 	r->content_transfer_encoding=p;
@@ -870,6 +891,8 @@ char	*p;
 	/* Everything up to the 1st ; is the content type */
 
 	p=lower_paste_tokens(header, 2, start-2);
+	if (!p)	return;
+
 	(*init_token)(r, p);
 	if (start < header->ntokens) start++;
 
@@ -895,6 +918,7 @@ char	*p;
 			{
 				++j;
 				p=lower_paste_token(header, start);
+				if (!p)	return;
 				(*init_parameter)(r, p, header, j, i-j);
 				free(p);
 			}
@@ -931,6 +955,8 @@ char	*p;
 	p=strcmp(name, "charset") == 0 ?
 			lower_paste_tokens(header, start, len):
 			paste_tokens(header, start, len);
+	if (!p)	return;
+
 	rfc2045_setattr(&r->content_type_attr, name, p);
 	free(p);
 
@@ -969,6 +995,8 @@ static void save_content_disposition_parameter(
 char	*p;
 
 	p=paste_tokens(header, start, len);
+	if (!p)	return;
+
 	rfc2045_setattr(&r->content_disposition_attr, name, p);
 	free(p);
 }
@@ -983,12 +1011,18 @@ int	i;
 	if (!cb || !*cb)	return (0);
 
 	t=rfc822t_alloc(cb, 0);
-	if (!t)			rfc2045_enomem();
+	if (!t)
+	{
+		rfc2045_enomem();
+		return(0);
+	}
+
 	a=rfc822a_alloc(t);
 	if (!a)
 	{
 		rfc822t_free(t);
 		rfc2045_enomem();
+		return (0);
 	}
 	for (i=0; i<a->naddrs; i++)
 		if (a->addrs[i].tokens)
@@ -1012,7 +1046,12 @@ static void content_id(struct rfc2045 *p, struct rfc822t *t)
 struct	rfc822a	*a=rfc822a_alloc(t);
 int	i;
 
-	if (!a)	rfc2045_enomem();
+	if (!a)
+	{
+		rfc2045_enomem();
+		return;
+	}
+
 	for (i=0; i<a->naddrs; i++)
 		if (a->addrs[i].tokens)
 		{
@@ -1022,6 +1061,7 @@ int	i;
 			{
 				rfc822a_free(a);
 				rfc2045_enomem();
+				return;
 			}
 			if (p->content_id)
 				free(p->content_id);
@@ -1109,7 +1149,12 @@ void rfc2045_setdefaultcharset(const char *charset)
 {
 char	*p=strdup(charset);
 
-	if (!p)	rfc2045_enomem();
+	if (!p)
+	{
+		rfc2045_enomem();
+		return;
+	}
+
 	if (rfc2045_defcharset)	free(rfc2045_defcharset);
 	rfc2045_defcharset=p;
 }
