@@ -49,10 +49,11 @@
 #include	"tcpremoteinfo.h"
 #include	"authlib/auth.h"
 #include	"numlib/numlib.h"
+#include	"argparse.h"
 
 #include	<netdb.h>
 
-static const char rcsid[]="$Id: tcpd.c,v 1.16 2000/06/04 18:58:21 mrsam Exp $";
+static const char rcsid[]="$Id: tcpd.c,v 1.17 2000/07/09 16:31:24 mrsam Exp $";
 
 static const char *accessarg=0;
 static const char *listenarg=0;
@@ -72,12 +73,6 @@ static const char *proxyarg=0;
 
 static void setup_block(const char *);
 
-struct args {
-	const char *name;
-	const char **valuep;
-	void (*funcp)(const char *);
-	} ;
-
 static struct args arginfo[]={
 	{"access", &accessarg},
 	{"address", &ipaddrarg},
@@ -223,46 +218,6 @@ static RETSIGTYPE sighup(int n)
 #endif
 }
 
-static int doargs(int argc, char **argv, struct args *s)
-{
-int	argn=1;
-int	i;
-
-	while (argn < argc)
-	{
-	const char *p;
-	int	l=0;
-
-		if ( argv[argn][0] != '-')	break;
-		if ( argv[argn][1] == 0)
-		{
-			++argn;
-			break;
-		}
-		for (i=0; s[i].name; i++)
-		{
-			l=strlen(s[i].name);
-			if (strncmp(s[i].name, argv[argn]+1, l) == 0 &&
-				(argv[argn][l+1] == 0 ||
-				argv[argn][l+1] == '='))	break;
-		}
-		if (s[i].name == 0)
-		{
-			fprintf(stderr, "%s: Invalid option: %s\n",
-				argv[0], argv[argn]);
-			_exit(1);
-		}
-		p=argv[argn]+1+l;
-		if (*p)	++p;
-		if (s[i].valuep)
-			*s[i].valuep=p;
-		else
-			(*s[i].funcp)(p);
-		++argn;
-	}
-	return (argn);
-}
-
 static int init(int argc, char **argv)
 {
 int	argn;
@@ -291,7 +246,7 @@ int	forced=0;
 	for (i=3; i<256; i++)
 		close(i);
 
-	argn=doargs(argc, argv, arginfo);
+	argn=argparse(argc, argv, arginfo);
 
 	if (argc - argn < 2)
 	{
@@ -802,7 +757,7 @@ int	pidptr;
 	RFC1035_ADDR addr;
 	int	addrport;
 	int	sinl;
-#ifdef	SOL_LINGER
+#ifdef	SO_LINGER
 	struct	linger l;
 #endif
 	pid_t	p;
@@ -849,19 +804,19 @@ int	pidptr;
 		}
 
 		dummy=1;
-#ifdef	SOL_KEEPALIVE
-		if (setsockopt(sockfd, SOL_SOCKET, SOL_KEEPALIVE,
+#ifdef	SO_KEEPALIVE
+		if (setsockopt(sockfd, SOL_SOCKET, SO_KEEPALIVE,
 			(const char *)&dummy, sizeof(dummy)) < 0)
 		{
 			perror("setsockopt");
 		}
 #endif
 
-#ifdef	SOL_LINGER
+#ifdef	SO_LINGER
 		l.l_onoff=0;
 		l.l_linger=0;
 
-		if (setsockopt(sockfd, SOL_SOCKET, SOL_LINGER,
+		if (setsockopt(sockfd, SOL_SOCKET, SO_LINGER,
 			(const char *)&l, sizeof(l)) < 0)
 		{
 			perror("setsockopt");
